# in this file, special algorithms are applied to avoid
# failure detecting obstacle from the edge
from time import sleep
from threading import Thread

HISTORY_LEN = 10  # keep a history list of this length
IGNORE_MOST_RECENT = 4  # a integer n. when compare the now to the prev, ignore the n most recent records
INTERVAL = 0.02  # a float t which MUST BE > 0.01. Refresh distance every t seconds
_debug = True  # if broadcast debug info


class Distance:
    _lock = False

    def debug(*args):
        if _debug:
            print "DIST: ", args

    def _monitor(self):
        sonic = self.sensor
        print('DIST: initializing, may take 1 more second')
        sleep(1)
        sleep_interval = INTERVAL - 0.01  # the sonic.get_distance method takes 0.01s to run
        debug = self.debug

        while True:
            sleep(sleep_interval)
            if self._lock:
                continue
            self.now = sonic.get_distance()
            self.history.pop()
            self.history.insert(0, self.now)

            self.prev = sum(self.history[IGNORE_MOST_RECENT:]) / (HISTORY_LEN - IGNORE_MOST_RECENT)
            _delta = self.now - self.prev
            _threshold = 10

            if _delta > 100:  # approaching but not in an angle
                self.danger = True
                debug("great gap detected. Danger! ")
                _threshold = 200
                debug("threshold set to 200")
            elif self.now < _threshold:  # approaching and heading to target
                self.danger = True
                debug("facing object that is too close. Danger! ")
            elif self.danger and self.now > _threshold:
                self.danger = False
                debug("danger alert disarmed")
            self.reset()

    def activate(self):
        work = Thread(target=self._monitor, args=(self,))
        work.start()

    def reset(self):
        self._lock = True
        sensor = self.sensor
        self.now = sensor.get_distance()
        self.history = [sensor.get_distance() for i in range(HISTORY_LEN)]
        self._lock = False
        return self.history

    def __init__(self, sensor, obstacle_inbound=False):
        self.sensor = sensor
        self.now = sensor.get_distance()
        self.history = [sensor.get_distance() for i in range(HISTORY_LEN)]
        self.danger = obstacle_inbound
        self.activate()

    def __call__(self):
        return self.now



from setuptools import setup

setup(
    # to be displayed on PyPI
    name='klego',
    version='0.98.1',
    description='An easy-to-go, yet specific and powerful'
                ' python package for Lego NXT',
    long_description='An easy-to-go'
                ' python package for Lego NXT control'
                ' built especially for popular Lego tasks'
                ' like line-following, bonus block hitting.'
                ' More feature with ML or DL might be included'
                ' in the future as well',
    keywords='python lego nxt mindstorm easy',
    project_urls={
        'Documentation': 'https://github.com/KivenChen/PyLego',
        'Source Code': 'https://github.com/KivenChen/PyLego',
        'More by Kiven': "https://github.com/Kivenchen",
    },
    author='Kiven',
    author_email='sdckivenchen@gmail.com',
    url='https://kivenchen,us',

    packages=['klego'],
    install_requires=['pyusb', 'nxt-python', 'pybluez==0.22k0'],
    dependency_links=[
        'https://kivenchen.us/kiven-s-pybluez/PyBluez-0.22k0.tar.gz'],
    zip_safe=False,
    test_suite='nose.collector',
    tests_require=['nose'],
)



from math import sin, sqrt, cos, radians as rad

'''
this file provides the utilities for tracking robot and boxes locations
the **core** module will import this file and create *boxes* and *pos* for use
I will consider separating this util files from *nav_utils*
so that the core tech of coordinating will not be open-sourced
'''

# forward and left when see dark
# forward and right when see light


class Position:
	def __init__(self, x=0, y=0, d=0):
		self.x = x
		self.y = y
		self.d = d
		self.accurate = True
	
	def track(self, d_delta, dpm, continuous=False):
		# tracks the position change
		# d_delta: the change of direction clockwise
		# dpm: the displacement x
		# continuous: handle f() and b() separately
		if continuous:
			pass
		self.d += d_delta
		dx = dpm * sin(rad(self.d))
		dy = dpm * cos(rad(self.d))
		self.x += dx
		self.y += dy
		return self

	def dist(self, pos):
		# output distance
		return sqrt((self.x-pos.x)**2+(self.y - pos.y)**2)

	def __str__(self):
		return '%d, %d, direction: %d' % (self.x, self.y, self.d)


class Box(Position):
	def __init__(self, x=0, y=0, d=0, ok=True):
		self.x = x
		self.y = y
		self.d = d
		self.new = ok  # marks if this is ok-to-go


class Boxes(list):
	def __init__(self):
		super(Boxes, self).__init__()

	def add(self, inst, new=True):
		if isinstance(inst, Position):
			inst = Box(inst.x, inst.y, inst.d, new)
		self.append(inst)
		return self
	
	def nearest(self, robo_pos):
		# return the nearest and ok-to-go box
		min = 0
		mindist = 99999
		for i, box in enumerate(self):
			dist = robo_pos.dist(box.pos)
			if dist < mindist and box.new:
				mindist = dist
				min = box
		return min

	def overlapped(self, target_pos, threshold=14):
		# check if the target is already discovered
		if type(target_pos) == Box:
			target_pos = target_pos.pos
		for i, box in enumerate(self):
			if box.dist(target_pos) <= threshold:
				return True
		return False



def test():
	b1 = Box(10, 20)
	b2 = Box(10, 20)
	print(b2.x)
	boxes = Boxes()
	boxes.add(b1)
	boxes.add(b2)
	print(boxes[1])
	print(boxes._discover(Position(30, 15, 45), 5))
	print(boxes)


if __name__ == "__main__":
	test()
